@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2003 Zope Corporation and Contributors.
+# Copyright (c) 2003 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -177,7 +177,7 @@ class ClientCache(object):
 
         # tid for the most recent transaction we know about.  This is also
         # stored near the start of the file.
-        self.tid = None
+        self.tid = z64
 
         # Always the offset into the file of the start of a block.
         # New and relocated objects are always written starting at
@@ -207,7 +207,25 @@ class ClientCache(object):
             self.f.write(magic+z64)
             logger.info("created temporary cache file %r", self.f.name)
 
-        self._initfile(fsize)
+        try:
+            self._initfile(fsize)
+        except:
+            self.f.close()
+            if not path:
+                raise # unrecoverable temp file error :(
+            badpath = path+'.bad'
+            if os.path.exists(badpath):
+                logger.critical(
+                    'Removing bad cache file: %r (prev bad exists).',
+                    path, exc_info=1)
+                os.remove(path)
+            else:
+                logger.critical('Moving bad cache file to %r.',
+                                badpath, exc_info=1)
+                os.rename(path, badpath)
+            self.f = open(path, 'wb+')
+            self.f.write(magic+z64)
+            self._initfile(ZEC_HEADER_SIZE)
 
         # Statistics:  _n_adds, _n_added_bytes,
         #              _n_evicts, _n_evicted_bytes,
@@ -379,10 +397,13 @@ class ClientCache(object):
     # Close the underlying file.  No methods accessing the cache should be
     # used after this.
     def close(self):
-        if self.f:
-            sync(self.f)
-            self.f.close()
-            self.f = None
+        self._unsetup_trace()
+        f = self.f
+        self.f = None
+        if f is not None:
+            sync(f)
+            f.close()
+
         if hasattr(self,'_lock_file'):
             self._lock_file.close()
 
@@ -435,10 +456,14 @@ class ClientCache(object):
     # recent tid.
     @locked
     def setLastTid(self, tid):
-        if (self.tid is not None) and (tid <= self.tid) and self:
+        if (not tid) or (tid == z64):
+            return
+        if (tid <= self.tid) and self._len:
+            if tid == self.tid:
+                return                  # Be a little forgiving
             raise ValueError("new last tid (%s) must be greater than "
-                             "previous one (%s)" % (u64(tid),
-                                                    u64(self.tid)))
+                             "previous one (%s)"
+                             % (u64(tid), u64(self.tid)))
         assert isinstance(tid, str) and len(tid) == 8, tid
         self.tid = tid
         self.f.seek(len(magic))
@@ -448,13 +473,9 @@ class ClientCache(object):
     ##
     # Return the last transaction seen by the cache.
     # @return a transaction id
-    # @defreturn string, or None if no transaction is yet known
+    # @defreturn string, or 8 nulls if no transaction is yet known
     def getLastTid(self):
-        tid = self.tid
-        if tid == z64:
-            return None
-        else:
-            return tid
+        return self.tid
 
     ##
     # Return the current data record for oid.
@@ -482,7 +503,7 @@ class ClientCache(object):
         assert len(data) == ldata, (ofs, self.f.tell(), oid, len(data), ldata)
 
         # WARNING: The following assert changes the file position.
-        # We must not depend on ths below or we'll fail in optimized mode.
+        # We must not depend on this below or we'll fail in optimized mode.
         assert read(8) == oid, (ofs, self.f.tell(), oid)
 
         self._n_accesses += 1
@@ -523,7 +544,7 @@ class ClientCache(object):
         assert len(data) == ldata, (ofs, self.f.tell())
 
         # WARNING: The following assert changes the file position.
-        # We must not depend on ths below or we'll fail in optimized mode.
+        # We must not depend on this below or we'll fail in optimized mode.
         assert read(8) == oid, (ofs, self.f.tell(), oid)
 
         if end_tid < before_tid:
@@ -639,23 +660,9 @@ class ClientCache(object):
     # - oid object id
     # - tid the id of the transaction that wrote a new revision of oid,
     #        or None to forget all cached info about oid.
-    # - server_invalidation, a flag indicating whether the
-    #       invalidation has come from the server. It's possible, due
-    #       to threading issues, that when applying a local
-    #       invalidation after a store, that later invalidations from
-    #       the server may already have arrived.
 
     @locked
-    def invalidate(self, oid, tid, server_invalidation=True):
-        if tid is not None:
-            if tid > self.tid:
-                self.setLastTid(tid)
-            elif tid < self.tid:
-                if server_invalidation:
-                    raise ValueError("invalidation tid (%s) must not be less"
-                                     " than previous one (%s)" %
-                                     (u64(tid), u64(self.tid)))
-
+    def invalidate(self, oid, tid):
         ofs = self.current.get(oid)
         if ofs is None:
             # 0x10 == invalidate (miss)
@@ -677,6 +684,9 @@ class ClientCache(object):
             self._trace(0x1E, oid, tid)
             self._len -= 1
         else:
+            if tid == saved_tid:
+                logger.warning("Ignoring invalidation with same tid as current")
+                return
             self.f.seek(ofs+21)
             self.f.write(tid)
             self._set_noncurrent(oid, saved_tid, ofs)
@@ -727,6 +737,10 @@ class ClientCache(object):
     # self._tracefile.  If not, or we can't write to the trace file, disable
     # tracing by setting self._trace to a dummy function, and set
     # self._tracefile to None.
+    _tracefile = None
+    def _trace(self, *a, **kw):
+        pass
+
     def _setup_trace(self, path):
         _tracefile = None
         if path and os.environ.get("ZEO_CACHE_TRACE"):
@@ -739,7 +753,6 @@ class ClientCache(object):
                 logger.info("opened tracefile %r", tfn)
 
         if _tracefile is None:
-            self._trace = lambda *a, **k: None
             return
 
         now = time.time
@@ -748,7 +761,7 @@ class ClientCache(object):
             # The first hex digit shows the operation, the second the outcome.
             # This method has been carefully tuned to be as fast as possible.
             # Note: when tracing is disabled, this method is hidden by a dummy.
-            encoded = (dlen + 255) & 0x7fffff00 | code
+            encoded = (dlen << 8) + code
             if tid is None:
                 tid = z64
             if end_tid is None:
@@ -763,8 +776,15 @@ class ClientCache(object):
                 raise
 
         self._trace = _trace
+        self._tracefile = _tracefile
         _trace(0x00)
 
+    def _unsetup_trace(self):
+        if self._tracefile is not None:
+            del self._trace
+            self._tracefile.close()
+            del self._tracefile
+
 def sync(f):
     f.flush()
 
