@@ -13,11 +13,9 @@
 ##############################################################################
 
 import os
-import sys
 import time
-import random
+import socket
 import asyncore
-import tempfile
 import threading
 import logging
 
@@ -33,6 +31,7 @@ from ZODB.tests.StorageTestBase import StorageTestBase
 from ZODB.tests.MinPO import MinPO
 from ZODB.tests.StorageTestBase \
      import zodb_pickle, zodb_unpickle, handle_all_serials, handle_serials
+import ZODB.tests.util
 
 import transaction
 from transaction import Transaction
@@ -93,7 +92,7 @@ class CommonSetupTearDown(StorageTestBase):
     monitor = 0
     db_class = DummyDB
 
-    def setUp(self):
+    def setUp(self, before=None):
         """Test setup for connection tests.
 
         This starts only one server; a test may start more servers by
@@ -102,7 +101,7 @@ class CommonSetupTearDown(StorageTestBase):
         """
         self.__super_setUp()
         logging.info("setUp() %s", self.id())
-        self.file = tempfile.mktemp()
+        self.file = 'storage_conf'
         self.addr = []
         self._pids = []
         self._servers = []
@@ -111,9 +110,17 @@ class CommonSetupTearDown(StorageTestBase):
         self._newAddr()
         self.startServer()
 
+#         self._old_log_level = logging.getLogger().getEffectiveLevel()
+#         logging.getLogger().setLevel(logging.WARNING)
+#         self._log_handler = logging.StreamHandler()
+#         logging.getLogger().addHandler(self._log_handler)
+
     def tearDown(self):
         """Try to cause the tests to halt"""
-        logging.info("tearDown() %s" % self.id())
+#         logging.getLogger().setLevel(self._old_log_level)
+#         logging.getLogger().removeHandler(self._log_handler)
+#         logging.info("tearDown() %s" % self.id())
+
         for p in self.conf_paths:
             os.remove(p)
         if getattr(self, '_storage', None) is not None:
@@ -125,15 +132,16 @@ class CommonSetupTearDown(StorageTestBase):
         for adminaddr in self._servers:
             if adminaddr is not None:
                 forker.shutdown_zeo_server(adminaddr)
-        if hasattr(os, 'waitpid'):
-            # Not in Windows Python until 2.3
-            for pid in self._pids:
+        for pid in self._pids:
+            try:
                 os.waitpid(pid, 0)
+            except OSError:
+                pass # The subprocess module may already have waited
+                    
         for c in self.caches:
             for i in 0, 1:
                 for ext in "", ".trace", ".lock":
-                    base = "%s-%s.zec%s" % (c, "1", ext)
-                    path = os.path.join(tempfile.tempdir, base)
+                    path = "%s-%s.zec%s" % (c, "1", ext)
                     # On Windows before 2.3, we don't have a way to wait for
                     # the spawned server(s) to close, and they inherited
                     # file descriptors for our open files.  So long as those
@@ -158,7 +166,7 @@ class CommonSetupTearDown(StorageTestBase):
         self.addr.append(self._getAddr())
 
     def _getAddr(self):
-        return 'localhost', forker.get_port()
+        return 'localhost', forker.get_port(self)
 
     def getConfig(self, path, create, read_only):
         raise NotImplementedError
@@ -174,7 +182,7 @@ class CommonSetupTearDown(StorageTestBase):
         self.caches.append(cache)
         storage = TestClientStorage(self.addr,
                                     client=cache,
-                                    var=tempfile.tempdir,
+                                    var='.',
                                     cache_size=cache_size,
                                     wait=wait,
                                     min_disconnect_poll=0.1,
@@ -583,15 +591,15 @@ class InvqTests(CommonSetupTearDown):
     invq = 3
 
     def checkQuickVerificationWith2Clients(self):
-        perstorage = self.openClientStorage(cache="test")
-        self.assertEqual(perstorage.verify_result, "full verification")
+        perstorage = self.openClientStorage(cache="test", cache_size=4000)
+        self.assertEqual(perstorage.verify_result, "empty cache")
 
         self._storage = self.openClientStorage()
         oid = self._storage.new_oid()
         oid2 = self._storage.new_oid()
         # When we create a new storage, it should always do a full
         # verification
-        self.assertEqual(self._storage.verify_result, "full verification")
+        self.assertEqual(self._storage.verify_result, "empty cache")
         # do two storages of the object to make sure an invalidation
         # message is generated
         revid = self._dostore(oid)
@@ -601,17 +609,22 @@ class InvqTests(CommonSetupTearDown):
         revid2 = self._dostore(oid2)
         revid2 = self._dostore(oid2, revid2)
 
-        # sync() is needed to prevent invalidation for oid from arriving
-        # in the middle of the load() call.
-        perstorage.sync()
         perstorage.load(oid, '')
         perstorage.close()
 
+        forker.wait_until(lambda : os.path.exists('test-1.zec'))
+
         revid = self._dostore(oid, revid)
+
         perstorage = self.openClientStorage(cache="test")
+
+        forker.wait_until(
+            (lambda : perstorage.verify_result == "quick verification"),
+            onfail=(lambda : None))
+
         self.assertEqual(perstorage.verify_result, "quick verification")
         self.assertEqual(perstorage._server._last_invals,
-                         (revid, [(oid, '')]))
+                         (revid, [oid]))
 
         self.assertEqual(perstorage.load(oid, ''),
                          self._storage.load(oid, ''))
@@ -619,18 +632,22 @@ class InvqTests(CommonSetupTearDown):
 
     def checkVerificationWith2ClientsInvqOverflow(self):
         perstorage = self.openClientStorage(cache="test")
-        self.assertEqual(perstorage.verify_result, "full verification")
+        self.assertEqual(perstorage.verify_result, "empty cache")
 
         self._storage = self.openClientStorage()
         oid = self._storage.new_oid()
         # When we create a new storage, it should always do a full
         # verification
-        self.assertEqual(self._storage.verify_result, "full verification")
+        self.assertEqual(self._storage.verify_result, "empty cache")
         # do two storages of the object to make sure an invalidation
         # message is generated
         revid = self._dostore(oid)
         revid = self._dostore(oid, revid)
-
+        forker.wait_until(
+            "Client has seen all of the transactions from the server",
+            lambda :
+            perstorage.lastTransaction() == self._storage.lastTransaction()
+            )
         perstorage.load(oid, '')
         perstorage.close()
 
@@ -827,7 +844,7 @@ class ReconnectionTests(CommonSetupTearDown):
         self._storage = self.openClientStorage()
         # When we create a new storage, it should always do a full
         # verification
-        self.assertEqual(self._storage.verify_result, "full verification")
+        self.assertEqual(self._storage.verify_result, "empty cache")
         self._dostore()
         self.shutdownServer()
         self.pollDown()
@@ -840,22 +857,24 @@ class ReconnectionTests(CommonSetupTearDown):
 
     def checkNoVerificationOnServerRestartWith2Clients(self):
         perstorage = self.openClientStorage(cache="test")
-        self.assertEqual(perstorage.verify_result, "full verification")
+        self.assertEqual(perstorage.verify_result, "empty cache")
 
         self._storage = self.openClientStorage()
         oid = self._storage.new_oid()
         # When we create a new storage, it should always do a full
         # verification
-        self.assertEqual(self._storage.verify_result, "full verification")
+        self.assertEqual(self._storage.verify_result, "empty cache")
         # do two storages of the object to make sure an invalidation
         # message is generated
         revid = self._dostore(oid)
-        self._dostore(oid, revid)
-
+        revid = self._dostore(oid, revid)
+        forker.wait_until(
+            "Client has seen all of the transactions from the server",
+            lambda :
+            perstorage.lastTransaction() == self._storage.lastTransaction()
+            )
         perstorage.load(oid, '')
-
         self.shutdownServer()
-
         self.pollDown()
         self._storage.verify_result = None
         perstorage.verify_result = None
@@ -917,6 +936,7 @@ class ReconnectionTests(CommonSetupTearDown):
         self.shutdownServer()
         logging.info("checkReconnection(): About to restart server")
         self.startServer(create=0)
+        forker.wait_until('reconnect', self._storage.is_connected)
         oid = self._storage.new_oid()
         obj = MinPO(12)
         while 1:
@@ -972,6 +992,16 @@ class TimeoutTests(CommonSetupTearDown):
         storage.tpc_vote(txn)
         time.sleep(2)
         self.assertRaises(ClientDisconnected, storage.tpc_finish, txn)
+
+        # Make sure it's logged as CRITICAL
+        for line in open("server-%s.log" % self.addr[0][1]):
+            if (('Transaction timeout after' in line) and
+                ('CRITICAL ZEO.StorageServer' in line)
+                ):
+                break
+        else:
+            self.assert_(False, 'bad logging')
+
         storage.close()
 
     def checkTimeoutOnAbort(self):
@@ -1078,7 +1108,8 @@ class TimeoutTests(CommonSetupTearDown):
         self.assertRaises(ConflictError, storage.tpc_vote, t)
         # Even aborting won't help.
         storage.tpc_abort(t)
-        storage.tpc_finish(t)
+        self.assertRaises(ZODB.POSException.StorageTransactionError,
+                          storage.tpc_finish, t)
         # Try again.
         obj.value = 10
         t = Transaction()
@@ -1088,7 +1119,7 @@ class TimeoutTests(CommonSetupTearDown):
         self.assertRaises(ConflictError, storage.tpc_vote, t)
         # Abort this one and try a transaction that should succeed.
         storage.tpc_abort(t)
-        storage.tpc_finish(t)
+        
         # Now do a store.
         obj.value = 11
         t = Transaction()
@@ -1175,3 +1206,18 @@ class MSTThread(threading.Thread):
                 c.close()
             except:
                 pass
+
+# Run IPv6 tests if V6 sockets are supported
+try:
+    socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
+except (socket.error, AttributeError):
+    pass
+else:
+    class V6Setup:
+        def _getAddr(self):
+            return '::1', forker.get_port(self)
+
+    _g = globals()
+    for name, value in _g.items():
+        if isinstance(value, type) and issubclass(value, CommonSetupTearDown):
+            _g[name+"V6"] = type(name+"V6", (V6Setup, value), {})
