@@ -12,15 +12,10 @@
 #
 ##############################################################################
 import asyncore
-import atexit
-import errno
-import select
 import sys
 import threading
 import logging
 
-import traceback, time
-
 import ZEO.zrpc.trigger
 
 from ZEO.zrpc import smac
