@@ -13,7 +13,7 @@
 ##############################################################################
 """Database connection support
 
-$Id: Connection.py 117934 2010-10-26 21:20:03Z jim $"""
+$Id: Connection.py 120291 2011-02-11 23:50:41Z jim $"""
 
 import logging
 import sys
@@ -21,7 +21,6 @@ import tempfile
 import threading
 import warnings
 import os
-import shutil
 import time
 
 from persistent import PickleCache
@@ -48,7 +47,7 @@ from ZODB.POSException import InvalidObjectReference, ConnectionStateError
 from ZODB.POSException import ConflictError, ReadConflictError
 from ZODB.POSException import Unsupported, ReadOnlyHistoryError
 from ZODB.POSException import POSKeyError
-from ZODB.serialize import ObjectWriter, ObjectReader, myhasattr
+from ZODB.serialize import ObjectWriter, ObjectReader
 from ZODB.utils import p64, u64, z64, oid_repr, positive_id
 from ZODB import utils
 
@@ -452,7 +451,6 @@ class Connection(ExportImport, object):
                 del obj._p_oid
                 if obj._p_changed:
                     obj._p_changed = False
-                self._db.save_oid(oid)
             else:
 
                 # Note: If we invalidate a non-ghostifiable object
@@ -760,7 +758,6 @@ class Connection(ExportImport, object):
             self._creating = {}
 
         for oid in creating:
-            self._db.save_oid(oid)
             o = self._cache.get(oid)
             if o is not None:
                 del self._cache[oid]
